/*
 * HCS API
 *
 * No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)
 *
 * API version: 2.1
 * Generated by: Swagger Codegen (https://github.com/swagger-api/swagger-codegen.git)
 */

package hcsschema

type VideoMonitor struct {

	HorizontalResolution int32 `json:"HorizontalResolution,omitempty"`

	VerticalResolution int32 `json:"VerticalResolution,omitempty"`

	ConnectionOptions *RdpConnectionOptions `json:"ConnectionOptions,omitempty"`
}
