/*
 * HCS API
 *
 * No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)
 *
 * API version: 2.1
 * Generated by: Swagger Codegen (https://github.com/swagger-api/swagger-codegen.git)
 */

package hcsschema

//  ComPort specifies the named pipe that will be used for the port, with empty string indicating a disconnected port.
type ComPort struct {

	NamedPipe string `json:"NamedPipe,omitempty"`

	OptimizeForDebugger bool `json:"OptimizeForDebugger,omitempty"`
}
