package version

// Version is the version of the build.
// NOTE: remember to bump the version at the top
// of the top-level README.md file when this is
// bumped.
const Version = "1.6.2"

// RemoteAPIVersion is the version for the remote
// client API.  It is used to determine compatibility
// between a remote podman client and its backend
const RemoteAPIVersion = 1
