// +build windows

package ocicni

const (
	// DefaultConfDir is the default place to look for CNI Network
	DefaultConfDir = "C:\\cni\\etc\\net.d"
	// DefaultBinDir is the default place to look for cni config files
	DefaultBinDir = "C:\\cni\\bin"
)
