// Copyright (C) 2016-2018 Vladimir Bauer
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

// Package mpb is a library for rendering progress bars in terminal applications.
package mpb
